function [models, dimension] = learnTDNNModels(timeSeries)

% [models dimension] = learnTDNNModels(timeSeries)
%
% Input Variables
%       timeSeries - A two dimensional cell array. The
%                   first cell is a text label for the second cell. 
%                   The second cell is a time series.
%
% Output Variables
%       models - the models for each class.
%       dimension - number of inputs from the time series
%
% Language - MATLAB
%  
% Description:  Builds TDNN models over signals for each class
%
% Created:     
%               Date: 7/1/2003  
%               By:  Richard J. Povinelli
%               Marquette University
%
% Modified:    
%               Version: 2.0
%               Date: 9/5/2003
%               By: Richard J. Povinelli
%               Why: add in code to determine dimension to make cpu time
%                    comparisions accurate.

[numberOfSignals dummy] = size(timeSeries);
% normalize each time series to zero mean and unit variance
for i = 1:numberOfSignals
  signal = timeSeries{i,2};
  signalMean = mean(signal);
  signalStd = std(signal);
  timeSeries{i,2} = (signal - signalMean)/signalStd;
end %for

timeLag = determineTimeLag(timeSeries(:,2));
dimension = determineDimension(timeSeries(:,2), timeLag); %use same dimension as RPS method
lags = createLags(dimension + 1,1); %use this to create the target vector also

allLabels = timeSeries(:,1); %copies the labels out of data
uniqueLabels = unique(allLabels); %find the unique class labels
numberOfClasses = length(uniqueLabels); %find the number of classes

models = cell(numberOfClasses,2);

for i = 1:numberOfClasses %form a reconstructed phase space and insert corresponding signals
  rps = []; %the embedding with all the signals
  classIndices = strmatch(uniqueLabels(i),allLabels,'exact'); %get indices for each class
  elementsInClass = length(classIndices);
  for j = 1:elementsInClass %build a RPS for the class
    rps = [rps embed(timeSeries{classIndices(j),2},lags)];
  end %for
  
  p = rps(1:dimension,:); %strip of the first dimension rows as the training
  t = rps(dimension + 1,:); %use the last row as the target
  
  uniqueLabels(i) %debug
  models(i,1) = uniqueLabels(i);
  
  %set up the tdnn
  tdnet = newff(minmax(p),[dimension,ceil(sqrt(dimension)),1],{'tansig','tansig','purelin'},'trainbfg');
  tdnet.trainParam.show = 5;
  tdnet.trainParam.epochs = 25;
  %tdnet.trainParam.mem_reduc = 4; %reduce lm memory use by about this factor  
  [tdnet] = train(tdnet,p,t);
  models(i,2) = {tdnet};
end %for

return